﻿Imports System.IO
Imports System.Runtime.InteropServices

Public Class Main

    <DllImport("user32.dll", EntryPoint:="FlashWindow")> _
    Public Shared Function FlashWindow(ByVal hwnd As Integer, ByVal bInvert As Integer) As Integer
    End Function

    Dim Calibrating As Boolean = False
    Dim Pause As Boolean = False
    Dim Mute As Boolean = False
    Dim MyPorts As Array
    Dim intCountDown As Integer = 1
    Dim intFlash As Integer = 5
    Dim InTray As Boolean = False

    Private Sub Main_Load(sender As Object, e As EventArgs) Handles MyBase.Load

        CmdArgs()

        PortsRefresh()
        Me.Icon = My.Resources.Eye_blue
        NotifyIcon1.Icon = My.Resources.Eye_blue
        My.Resources.Eye_blue.Dispose()

        'Set form elements
        StatusStrip1.SizingGrip = False

        'Restore previous settings
        chkNotificationsEnable.Checked = My.Settings.Notifications
        chkMinToTray.Checked = My.Settings.MinToTray
        chkAutoConnect.Checked = My.Settings.AutoConnect
        cboPorts.SelectedItem = My.Settings.SensorPort
        txtMessage.Text = My.Settings.Message
        numDelay.Value = My.Settings.Delay
        chkAudioEnable.Checked = My.Settings.Audio
        chkAudioCustomWav.Checked = My.Settings.CustomWAV
        txtCustomWav.Text = My.Settings.WAVFile
        chkFlashTaskbar.Checked = My.Settings.Flash
        chkRunEnable.Checked = My.Settings.Run
        txtRunCmd.Text = My.Settings.Command
        chkHideCmd.Checked = My.Settings.HideCmd

        If chkAutoConnect.Checked = True Then
            ConnectPort(cboPorts.Text)
        End If

        If chkMinToTray.Checked = True Then
            TrayToggle("IN")
        End If

        ToggleControls(grpAudio, chkAudioEnable)
        ToggleControls(grpNotifications, chkNotificationsEnable)
        ToggleControls(grpRun, chkRunEnable)

    End Sub

    Private Sub CmdArgs()
        For Each arg As String In My.Application.CommandLineArgs
            Select Case arg.ToLower
                Case "/test"
                    grpTesting.Visible = True
                    grpTesting.Enabled = True
                    TestToolStripMenuItem.Enabled = True
                    TestToolStripMenuItem.Visible = True
                    Me.Size = New Size(285, 500)
                Case "/?"
                    Dim strHelp As String = "USBEye [/test][/?]" & vbCrLf & _
                                         vbCrLf & _
                                         "/test" & vbTab & "Enable testing functions." & vbCrLf & _
                                         "/?" & vbTab & "Display this help window."

                    MsgBox(strHelp, MsgBoxStyle.Information)
            End Select
        Next
    End Sub

    Private Sub PortsRefresh()
        MyPorts = System.IO.Ports.SerialPort.GetPortNames()
        Array.Sort(MyPorts)
        cboPorts.DataSource = MyPorts
    End Sub

    Private Delegate Sub MotionLEDVisibleInvolker(LEDName As ToolStripStatusLabel, State As Boolean)
    Private Sub MotionLEDVisible(LEDName As ToolStripStatusLabel, State As Boolean)
        If Me.InvokeRequired Then
            Me.Invoke(New MotionLEDVisibleInvolker(AddressOf MotionLEDVisible), LEDName, State)
        Else
            If State = False Then
                LEDName.ForeColor = Color.FromKnownColor(KnownColor.Control)
            Else
                LEDName.ForeColor = Color.Red
            End If
        End If
    End Sub

    Private Delegate Sub MotionDetectedInvolker()
    Private Sub MotionDetected()
        If Me.InvokeRequired Then
            Me.Invoke(New MotionDetectedInvolker(AddressOf MotionDetected))
        Else
            If chkNotificationsEnable.Checked = True And Pause = False Then
                NotifyIcon1.ShowBalloonTip(10000, "", txtMessage.Text & vbCrLf & Now.ToString("ddd h:mm tt"), ToolTipIcon.Warning)
                ToolStripStatusLabel1.Text = txtMessage.Text & " " & Now.ToString("ddd h:mm tt")
                timerStatus.Start()
            End If

            If chkAudioEnable.Checked = True And Mute = False Then
                Select Case chkAudioCustomWav.Checked
                    Case True
                        My.Computer.Audio.Play(txtCustomWav.Text)
                    Case False
                        System.Media.SystemSounds.Beep.Play()
                End Select
            End If

            If chkFlashTaskbar.Checked = True Then
                FlashWindow(Me.Handle, 1)
                timerFlash.Start()
            End If

            If chkRunEnable.Checked = True Then
                Dim CMD As New ProcessStartInfo
                With CMD
                    .FileName = txtRunCmd.Text
                    If chkHideCmd.Checked = True Then
                        .WindowStyle = ProcessWindowStyle.Hidden
                    End If
                End With

                If File.Exists(CMD.FileName.ToString) Then
                    Process.Start(CMD)
                Else
                    MsgBox("Run Command:  File not found!", MsgBoxStyle.Exclamation)
                End If

            End If

        End If
    End Sub

    Private Sub PauseToggle(Optional OnOFF As String = "")
        ' Handle Optional params
        OnOFF = OnOFF.ToUpper
        Select Case OnOFF
            Case "ON"
                Pause = False
            Case "OFF"
                Pause = True
        End Select

        Select Case Pause
            Case True 'Was paused so resume
                Pause = False
                btnPause.BackColor = Color.FromKnownColor(KnownColor.Control)
                PauseToolStripMenuItem.Checked = False
            Case False
                Pause = True 'Wasn't paused so pause
                btnPause.BackColor = Color.FromKnownColor(KnownColor.ControlDark)
                PauseToolStripMenuItem.Checked = True
        End Select
    End Sub

    Private Sub MuteToggle(Optional OnOFF As String = "")
        ' Handle Optional params
        OnOFF = OnOFF.ToUpper
        Select Case OnOFF
            Case "ON"
                Mute = False
            Case "OFF"
                Mute = True
        End Select

        Select Case Mute
            Case True 'Was muted so unmute
                Mute = False
                btnMute.BackColor = Color.FromKnownColor(KnownColor.Control)
                MuteToolStripMenuItem.Checked = False
            Case False
                Mute = True 'Wasn't muted so mute
                btnMute.BackColor = Color.FromKnownColor(KnownColor.ControlDark)
                MuteToolStripMenuItem.Checked = True
        End Select
    End Sub

    Private Sub CalibrateToggle(Optional OnOFF As String = "")
        ' Handle Optional params
        OnOFF = OnOFF.ToUpper
        Select Case OnOFF
            Case "ON"
                Calibrating = False
            Case "OFF"
                Calibrating = True
        End Select

        Select Case Calibrating
            Case True 'Was calibrating so stop
                Calibrating = False
                timerDelay.Stop()
                timerDelay.Dispose()
            Case False 'Wasn't calibrating so start
                Calibrating = True
                intCountDown = 0
                timerDelay.Interval = 1000
                timerDelay.Start()
                ToolStripStatusLabel1.Text = "Calibrating: " & numDelay.Value
        End Select
    End Sub

    Private Sub TrayToggle(Optional InOut As String = "")
        ' Handle Optional params
        InOut = InOut.ToUpper
        Select Case InOut
            Case "IN"
                InTray = False
            Case "OUT"
                InTray = True
        End Select

        Select Case InTray
            Case True 'Was hiding in tray so restore
                InTray = False
                Me.Show()
                Me.ShowInTaskbar = True
                Me.WindowState = FormWindowState.Normal
                HideUSBEyeToolStripMenuItem.Text = "Hide USBEye"
            Case False 'Wasn't in tray so hide it there
                InTray = True
                Me.WindowState = FormWindowState.Minimized
                Me.ShowInTaskbar = False
                Me.Hide()
                HideUSBEyeToolStripMenuItem.Text = "Show USBEye"
        End Select
    End Sub

    Private Sub timerDelay_Tick(sender As Object, e As EventArgs) Handles timerDelay.Tick
        Dim i As Integer = numDelay.Value - intCountDown

        If i > 0 Then
            ToolStripStatusLabel1.Text = "Calibrating: " & numDelay.Value - intCountDown
            intCountDown += 1
        Else
            CalibrateToggle("Off")
            ToolStripStatusLabel1.Text = " "
        End If
    End Sub

    Private Sub ConnectPort(MyComPort As String)
        If PortExists(MyComPort) = True Then
            SensorPort.PortName = MyComPort
            SensorPort.Open()

            If SensorPort.IsOpen Then
                btnConnect.Text = "Disconnect"
                ConnectToolStripMenuItem.Text = "Disconnect"
                ConnectLED.Visible = True
                CalibrateToggle("On")
            End If

        End If
    End Sub

    Private Sub DisconnectPort(MyComPort As String)
        btnConnect.Text = "Connect"
        ConnectToolStripMenuItem.Text = "Connect"
        ConnectLED.Visible = False
        MotionLEDVisible(MotionLED, False)
        NotifyIcon1.Icon = My.Resources.Eye_blue
        timerDelay.Stop()
        timerDelay.Dispose()
        ToolStripStatusLabel1.Text = ""

        If Calibrating = True Then
            CalibrateToggle("Off")
        End If

        If PortExists(MyComPort) = True And SensorPort.IsOpen Then
            SensorPort.Close()
        End If

        If Pause = True Then
            PauseToggle()
        End If
    End Sub

    Private Sub SensorPort_PinChanged(sender As Object, e As SerialPinChangedEventArgs) Handles SensorPort.PinChanged
        Dim Retry As Boolean = True

        While Retry = True
            Try
                If SensorPort.DsrHolding = True Then
                    MotionLEDVisible(MotionLED, True)
                    NotifyIcon1.Icon = My.Resources.Eye_red
                    My.Resources.Eye_red.Dispose()
                    If Calibrating = False Then
                        MotionDetected()
                    End If
                Else
                    MotionLEDVisible(MotionLED, False)
                    NotifyIcon1.Icon = My.Resources.Eye_blue
                    My.Resources.Eye_blue.Dispose()
                End If
                Retry = False
            Catch ex As System.IO.IOException
                If MessageBox.Show("Error: " & vbCrLf & ex.Message) = DialogResult.Retry Then
                    Retry = True
                Else
                    Retry = False
                    Application.Exit()
                End If
            End Try
        End While
    End Sub

    Private Sub ToggleControls(Grp As GroupBox, chkBox As CheckBox)
        For Each c As Control In Grp.Controls
            If Not c.Name = chkBox.Name Then
                c.Enabled = chkBox.Checked
            End If
        Next
    End Sub

    Private Sub Connect_Click(sender As Object, e As EventArgs) Handles btnConnect.Click, ConnectToolStripMenuItem.Click
        Select Case btnConnect.Text
            Case "Connect"
                ConnectPort(cboPorts.Text)
            Case "Disconnect"
                DisconnectPort(SensorPort.PortName)
        End Select
    End Sub

    Private Sub Pause_Click(sender As Object, e As EventArgs) Handles btnPause.Click, PauseToolStripMenuItem.Click
        PauseToggle()
    End Sub

    Private Sub Main_Resize(sender As Object, e As EventArgs) Handles MyBase.Resize
        If chkMinToTray.Checked = True Then
            Select Case Me.WindowState
                Case FormWindowState.Minimized
                    TrayToggle("IN")
                Case FormWindowState.Normal
                    TrayToggle("OUT")
            End Select
        End If
    End Sub

    Private Sub NotifyIcon1_MouseDoubleClick(sender As Object, e As MouseEventArgs) Handles NotifyIcon1.MouseDoubleClick
        TrayToggle()
    End Sub

    Private Sub NotifyIcon1_MouseClick(sender As Object, e As MouseEventArgs) Handles NotifyIcon1.MouseClick
        If e.Clicks = MouseButtons.Right Then
            NotifyIcon1.ContextMenuStrip.Show()
        End If
    End Sub

    Private Sub ExitToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles ExitToolStripMenuItem.Click
        Me.Close()
        Me.Dispose()
    End Sub

    Private Delegate Sub Main_FormClosingInvolker(sender As Object, e As FormClosingEventArgs)
    Private Sub Main_FormClosing(sender As Object, e As FormClosingEventArgs) Handles MyBase.FormClosing
        If Me.InvokeRequired Then
            Me.Invoke(New Main_FormClosingInvolker(AddressOf Main_FormClosing), sender, e)
        Else
            With My.Settings
                .MinToTray = chkMinToTray.Checked
                .Notifications = chkNotificationsEnable.Checked
                .AutoConnect = chkAutoConnect.Checked
                .SensorPort = cboPorts.SelectedItem.ToString
                .Message = txtMessage.Text
                .Delay = numDelay.Value
                .Audio = chkAudioEnable.Checked
                .CustomWAV = chkAudioCustomWav.Checked
                .WAVFile = txtCustomWav.Text
                .Flash = chkFlashTaskbar.Checked
                .Run = chkRunEnable.Checked
                .Command = txtRunCmd.Text
                .HideCmd = chkHideCmd.Checked
            End With
        End If
    End Sub

    Private Sub btnDebug_Click(sender As Object, e As EventArgs) Handles btnDebug.Click
        MotionDetected()
    End Sub

    Private Function PortExists(MyComPort As String) As Boolean
        Dim availablePorts As Array = SerialPort.GetPortNames
        Dim port As String = Array.IndexOf(availablePorts, MyComPort)
        If port = -1 Then
            Return False
        Else
            Return True
        End If
    End Function

    Private Sub btnRefresh_Click(sender As Object, e As EventArgs) Handles btnRefresh.Click
        PortsRefresh()
    End Sub

    Private Sub timerStatus_Tick(sender As Object, e As EventArgs) Handles timerStatus.Tick
        ToolStripStatusLabel1.Text = " "
        timerStatus.Stop()
        timerStatus.Dispose()
    End Sub

    Private Sub btnMute_Click(sender As Object, e As EventArgs) Handles btnMute.Click, MuteToolStripMenuItem.Click
        MuteToggle()
    End Sub

    Private Sub btnCustomWav_Click(sender As Object, e As EventArgs) Handles btnCustomWav.Click
        ofdCustomWav.ShowDialog()
    End Sub

    Private Sub ofdCustomWav_FileOk(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles ofdCustomWav.FileOk
        txtCustomWav.Text = ofdCustomWav.FileName
    End Sub

    Private Sub chkAudioEnable_CheckedChanged(sender As Object, e As EventArgs) Handles chkAudioEnable.CheckedChanged
        ToggleControls(grpAudio, chkAudioEnable)
    End Sub

    Private Sub chkNotifications_CheckedChanged(sender As Object, e As EventArgs) Handles chkNotificationsEnable.CheckedChanged
        ToggleControls(grpNotifications, chkNotificationsEnable)
    End Sub

    Private Sub chkRunEnable_CheckedChanged(sender As Object, e As EventArgs) Handles chkRunEnable.CheckedChanged, chkHideCmd.CheckedChanged
        ToggleControls(grpRun, chkRunEnable)
    End Sub

    Private Sub timerFlash_Tick(sender As Object, e As EventArgs) Handles timerFlash.Tick
        intFlash -= 1
        If intFlash > 0 Then
            FlashWindow(Me.Handle, 1)
        Else
            timerFlash.Stop()
            timerFlash.Dispose()
            intFlash = 5
        End If
    End Sub

    Private Sub btnSimMotion_Click(sender As Object, e As EventArgs) Handles btnSimMotion.Click, TestToolStripMenuItem.Click
        MotionDetected()
    End Sub

    Private Sub AboutToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles AboutToolStripMenuItem.Click
        frmAbout.Show()
    End Sub

    Private Sub HideUSBEyeToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles HideUSBEyeToolStripMenuItem.Click
        TrayToggle()
    End Sub
End Class